<?php defined('SYSPATH') or die('No direct access allowed.');

/**
 * This class provides Admin Email management
 *
 * @package    lnApp
 * @subpackage Page/Email
 * @category   Controllers
 * @author     Deon George
 * @copyright  (c) 2010 Deon George
 * @license    http://dev.leenooks.net/license.html
 */
class Controller_Admin_Email extends Controller_TemplateDefault_Admin {
	protected $secure_actions = array(
		'list'=>TRUE,
		'templateadd'=>TRUE,
		'templateedit'=>TRUE,
		'templatelist'=>TRUE,
	);

	/**
	 * Show a list of emails
	 */
	public function action_list() {
		Block::add(array(
			'title'=>_('System Emails Sent'),
			'body'=>Table::display(
				ORM::factory('Email_Log')->find_all(),
				25,
				array(
					'id'=>array('label'=>'ID','url'=>'user/email/view/'),
					'date_orig'=>array('label'=>'Date'),
					'email'=>array('label'=>'To'),
					'translate_resolve("subject")'=>array('label'=>'Subject'),
					'account->accnum()'=>array('label'=>'Cust ID'),
					'account->name()'=>array('label'=>'Customer'),
				),
				array(
					'page'=>TRUE,
					'type'=>'select',
					'form'=>'user/email/view',
				)),
			));
	}

	/**
	 * List our defined email templates
	 */
	public function action_templatelist() {
		$eto = ORM::factory('Email_Template');
		$output = '';

		// @todo Change this to use Table::
		$output .= View::factory($this->viewpath().'/head');
		foreach ($eto->find_all() as $et) {
			$output .= View::factory($this->viewpath().'/body')
				->set('template',$et);
		}
		$output .= View::factory($this->viewpath().'/foot');

		Block::add(array(
			'title'=>_('Available Email Templates'),
			'body'=>$output,
		));
	}

	/**
	 * Add a template
	 */
	public function action_templateadd() {
		$eto = ORM::factory('Email_Template');
		$output = '';

		if ($_POST AND $eto->values($_POST)->check()) {
			// @todo To update the setup ID
			$eto->email_setup_id=1;

			// Entry updated
			if ($eto->save()) {
				$x = $eto->email_template_translate->values($_POST['translate']['new']);

				$x->email_template_id = $eto->id;
				if ($x->check())
					$x->save();
			}
		}

		$output .= Form::open();
		$output .= View::factory($this->viewpath());
		$output .= View::factory($this->viewpath().'/translate');
		$output .= '<div>'.Form::submit('submit',_('Add'),array('class'=>'form_button')).'</div>';
		$output .= Form::close();

		Editor::add();
		Block::add(array(
			'title'=>_('Available Email Templates'),
			'body'=>$output,
		));
	}

	/**
	 * Edit Template Definition
	 * @todo Change this into an add_view function like payment()
	 */
	public function action_templateedit() {
		$id = $this->request->param('id');

		$eto = ORM::factory('Email_Template',$id);

		if (! $eto->loaded())
			HTTP::redirect('email/admin/template/list');

		$output = '';

		if ($_POST AND $eto->values($_POST)->check()) {
			// Entry updated
			if ($eto->save()) {
				foreach ($_POST['translate'] as $id => $details) {
					$x = $eto->email_template_translate->where('id','=',$id)->find();

					if ($x->values($details)->check())
						$x->save();
				}
			}
		}

		$output .= Form::open();

		$output .= View::factory($this->viewpath())
			->set('template',$eto);

		foreach ($eto->email_template_translate->find_all() as $to) {
			$output .= View::factory($this->viewpath().'/translate')
				->set('translate',$to);

			SystemMessage::add(array(
				'title'=>_('Available Variables'),
				'type'=>'info',
				'body'=>sprintf('This template uses the following TEXT variables (%s) and HTML variables (%s)',
					implode('|',array_values($to->variables('message_text'))),
					implode('|',array_values($to->variables('message_html')))),
			));
		}

		$output .= '<div>'.Form::submit('submit',_('Update'),array('class'=>'form_button')).'</div>';
		$output .= Form::close();

		Editor::add();
		Block::add(array(
			'title'=>sprintf(_('Edit Template '),$eto->name),
			'body'=>$output,
		));
	}
}
?>
